//User must modify the below package with their package name
package com.GPIODemo; 
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbAccessory;
import android.hardware.usb.UsbManager;
import android.os.Handler;
import android.os.Message;
import android.os.ParcelFileDescriptor;
import android.util.Log;



/******************************FT311 GPIO interface class******************************************/
public class FT311GPIOInterface extends Activity
{
	
	private static final String ACTION_USB_PERMISSION =    "com.GPIODemo.USB_PERMISSION";
	public UsbManager usbmanager;
	public UsbAccessory usbaccessory;
	public PendingIntent mPermissionIntent;
	public ParcelFileDescriptor filedescriptor;
	public FileInputStream inputstream;
	public FileOutputStream outputstream;
	public boolean mPermissionRequestPending = true;
	public handler_thread handlerThread;
	
	private byte [] usbdata; 
    private byte []	writeusbdata;
    private int readcount;
	
	
		
		/*constructor*/
	 public FT311GPIOInterface(Context context){
		 	super();
			/*shall we start a thread here or what*/
			usbdata = new byte[4]; 
	        writeusbdata = new byte[4];
	        
	        /***********************USB handling******************************************/
			
	        usbmanager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
	       // Log.d("LED", "usbmanager" +usbmanager);
	        mPermissionIntent = PendingIntent.getBroadcast(context, 0, new Intent(ACTION_USB_PERMISSION), 0);
	        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
	       filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
	       context.registerReceiver(mUsbReceiver, filter);
	        
	       inputstream = null;
	       outputstream = null;
		}
	 
	/*reset port*/
	 public void ResetPort()
	 {
		writeusbdata[0] = 0x14;
		writeusbdata[1] = 0x00;
		writeusbdata[2] = 0x00;
		writeusbdata[3] = 0x00;
			
		try {
			if(outputstream != null){
				outputstream.write(writeusbdata, 0,4);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		 
		 
	 }
	 
	 
		
		/*configure data*/
		public void ConfigPort(byte configOutMap, byte configINMap){
			configOutMap |= 0x80; 	// GPIO pin 7 is OUT
			configINMap &= 0x7F;	// GPIO pin 7 is OUT	
			writeusbdata[0] = 0x11;
			writeusbdata[1] = 0x00;
			writeusbdata[2] = configOutMap;
			writeusbdata[3] = configINMap;
			
			
				try {
					if(outputstream != null){
						outputstream.write(writeusbdata, 0,4);
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
		}
		/*write port*/
		public void WritePort(byte portData){
			portData |= 0x80; // GPIO pin 7 is high then LED is OFF
			writeusbdata[0] = 0x13;
			writeusbdata[1] = portData;
			writeusbdata[2] = 0x00;
			writeusbdata[3] = 0x00;
			
			
				try {
					if(outputstream != null){
						outputstream.write(writeusbdata, 0,4);
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
		}
		/*read port*/
		public byte ReadPort(){
			return usbdata[1];
			
		}
		
		
		/*resume accessory*/
		public void ResumeAccessory()
		{
			// Intent intent = getIntent();
			if (inputstream != null && outputstream != null) {
				return;
			}
			
			UsbAccessory[] accessories = usbmanager.getAccessoryList();
			UsbAccessory accessory = (accessories == null ? null : accessories[0]);
			if (accessory != null) {
				if (usbmanager.hasPermission(accessory)) {
					OpenAccessory(accessory);
				} 
				else
				{
					synchronized (mUsbReceiver) {
						if (!mPermissionRequestPending) {
							usbmanager.requestPermission(accessory,
									mPermissionIntent);
							mPermissionRequestPending = true;
						}
				}
			}
			} else {}

		}
		
		/*destroy accessory*/
		public void DestroyAccessory(){
			unregisterReceiver(mUsbReceiver);
			CloseAccessory();
		}
		

		
/*********************helper routines*************************************************/		
		
		public void OpenAccessory(UsbAccessory accessory)
		{
			filedescriptor = usbmanager.openAccessory(accessory);
			if(filedescriptor != null){
				usbaccessory = accessory;
				FileDescriptor fd = filedescriptor.getFileDescriptor();
				inputstream = new FileInputStream(fd);
				outputstream = new FileOutputStream(fd);
				/*check if any of them are null*/
				if(inputstream == null || outputstream==null){
					return;
				}
			}
			
			handlerThread = new handler_thread(handler, inputstream);
			handlerThread.start();
		}
		
		private void CloseAccessory()
		{
			try{
				if(filedescriptor != null)
					filedescriptor.close();
				
			}catch (IOException e){}
			
			try {
				if(inputstream != null)
						inputstream.close();
			} catch(IOException e){}
			
			try {
				if(outputstream != null)
						outputstream.close();
				
			}catch(IOException e){}
			/*FIXME, add the notfication also to close the application*/
			
			filedescriptor = null;
			inputstream = null;
			outputstream = null;
		
			System.exit(0);
		
		}
		
				
		/***********USB broadcast receiver*******************************************/
	    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() 
		{
			@Override
			public void onReceive(Context context, Intent intent) 
			{
				String action = intent.getAction();
				if (ACTION_USB_PERMISSION.equals(action)) 
				{
					synchronized (this)
					{
						UsbAccessory accessory = (UsbAccessory) intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
						if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false))
						{
							OpenAccessory(accessory);
						} 
						else 
						{
							Log.d("LED", "permission denied for accessory "+ accessory);
							
						}
						mPermissionRequestPending = false;
					}
				} 
				else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) 
				{
					UsbAccessory accessory = (UsbAccessory)intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
					if (accessory != null )//&& accessory.equals(usbaccessory))
					{
						CloseAccessory();
					}
				}else
				{
					Log.d("LED", "....");
				}
			}	
		};
	    
		
		


		final Handler handler =  new Handler()
	    {
	    	@Override 
	    	public void handleMessage(Message msg)
	    	{
	    		//
	    		
	    	}
	    	
	    };
		
		
		/*usb input data handler*/
		private class handler_thread  extends Thread {
			Handler mHandler;
			FileInputStream instream;
			
			handler_thread(Handler h,FileInputStream stream ){
				mHandler = h;
				instream = stream;
			}
			
			public void run()
			{
				
				while(true)
				{
					Message msg = mHandler.obtainMessage();
					try{
						if(instream != null)
						{	
						readcount = instream.read(usbdata,0,4);
						if(readcount > 0)
						{
							msg.arg1 = usbdata[0];
							msg.arg2 = usbdata[1];
						}
						mHandler.sendMessage(msg);
						}
						}catch (IOException e){}
				}
			}
		}
	    
		
		
	}