package com.UARTDemo;



import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.MotionEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.UARTDemo.R.drawable;

public class UARTDemoActivity extends Activity {
	
	/*thread to read the data*/
	public handler_thread handlerThread;
	
	/*declare a FT311 UART interface variable*/
    public FT311UARTInterface uartInterface;
    /*graphical objects*/
    EditText readText;
    EditText writeText;
    Spinner baudSpinner;;
    Spinner stopSpinner;
    Spinner dataSpinner;
    Spinner paritySpinner;
    Spinner flowSpinner;
    
    
    
    
    Button writeButton,configButton;
    
    /*local variables*/
    char [] writeBuffer;
    char[] readBuffer;
    byte [] actualNumBytes;
    
    byte numBytes;
    byte count;
    byte status;
    byte writeIndex = 0;
    byte readIndex =  0;
    
    int baudRate; /*baud rate*/
    byte stopBit; /*1:1stop bits, 2:2 stop bits*/
    byte dataBit; /*8:8bit, 7: 7bit*/
    byte parity;  /* 0: none, 1: odd, 2: even, 3: mark, 4: space*/
    byte flowControl; /*0:none, 1: flow control(CTS,RTS)*/
	
    
    
    
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        /*create editable text objects*/
        readText = (EditText)findViewById(R.id.ReadValues);
        readText.setInputType(0);
        writeText = (EditText)findViewById(R.id.WriteValues);
        
       
       configButton = (Button)findViewById(R.id.configButton);
        writeButton = (Button)findViewById(R.id.WriteButton);
        
        
        /*allocate buffer*/
        writeBuffer = new char[64];
        readBuffer = new char [64];
        actualNumBytes = new byte[1];
        
        
        
        /*setup the baud rate list*/
        baudSpinner = (Spinner)findViewById(R.id.baudRateValue);
        ArrayAdapter<CharSequence> baudAdapter = ArrayAdapter.createFromResource(this, R.array.baud_rate, 
        									android.R.layout.simple_spinner_item);
        baudAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        baudSpinner.setAdapter(baudAdapter);
        baudSpinner.setGravity(0x10);
        baudSpinner.setSelection(4);
        /*by default it is 9600*/
        baudRate = 9600;
        
        /*stop bits*/
        stopSpinner = (Spinner)findViewById(R.id.stopBitValue);
        ArrayAdapter<CharSequence> stopAdapter = ArrayAdapter.createFromResource(this,
        										 R.array.stop_bits, android.R.layout.simple_spinner_item);
        
        stopAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        stopSpinner.setAdapter(stopAdapter);
        stopSpinner.setGravity(0x01);
        /*default is stop bit 1*/
        stopBit = 1;
        
        
        /*daat bits*/
        dataSpinner = (Spinner)findViewById(R.id.dataBitValue);
        ArrayAdapter<CharSequence> dataAdapter = ArrayAdapter.createFromResource(this,
        										 R.array.data_bits, android.R.layout.simple_spinner_item);
        
        dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        dataSpinner.setAdapter(dataAdapter);
        dataSpinner.setGravity(0x11);
        dataSpinner.setSelection(1);
        /*default data bit is 8 bit*/
        dataBit = 8;
        
        /* parity*/
        paritySpinner = (Spinner)findViewById(R.id.parityValue);
        ArrayAdapter<CharSequence> parityAdapter = ArrayAdapter.createFromResource(this,
        										 R.array.parity, android.R.layout.simple_spinner_item);
        
        parityAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        paritySpinner.setAdapter(parityAdapter);
        paritySpinner.setGravity(0x11);
        /*default is none*/
        parity = 0;
        
        /*flow control*/
        flowSpinner = (Spinner)findViewById(R.id.flowControlValue);
        ArrayAdapter<CharSequence> flowAdapter = ArrayAdapter.createFromResource(this,
        										 R.array.flow_control, android.R.layout.simple_spinner_item);
        
        flowAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        flowSpinner.setAdapter(flowAdapter);
        flowSpinner.setGravity(0x11);
        /*default flow control is is none*/
        flowControl = 0;
        
        
       
        
        
        configButton.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				configButton.setBackgroundResource(drawable.start);
	
				uartInterface.SetConfig(baudRate, dataBit, stopBit, parity, flowControl);
					
				}
	
		});
        
        configButton.setOnTouchListener(new View.OnTouchListener() {
			
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				if(event.getAction()== MotionEvent.ACTION_DOWN)
						configButton.setBackgroundResource(drawable.start1);
				return false;
			}
       });
        
        
        /*handle write click*/
        writeButton.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				writeButton.setBackgroundResource(drawable.start);
				if(writeText.length() != 0x00)
				{
				
					numBytes = (byte)writeText.length();
					for(count=0;count<numBytes;count++){
						writeBuffer[count] = writeText.getText().charAt(count);
					}
					
					status = uartInterface.SendData(numBytes, writeBuffer);
				}
			
			}
		});

        writeButton.setOnTouchListener(new View.OnTouchListener() {
			
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				if(event.getAction()==MotionEvent.ACTION_DOWN)
					writeButton.setBackgroundResource(drawable.start1);
				return false;
			}
		});
        
        
        
        
        uartInterface = new FT311UARTInterface(this);
        
        
        handlerThread = new handler_thread(handler);
        handlerThread.start();
        
        
        
        /*set the adapter listeners for baud*/
        baudSpinner.setOnItemSelectedListener(new MyOnBaudSelectedListener());
        /*set the adapter listeners for stop bits*/
        stopSpinner.setOnItemSelectedListener(new MyOnStopSelectedListener());
        /*set the adapter listeners for data bits*/
        dataSpinner.setOnItemSelectedListener(new MyOnDataSelectedListener());
        /*set the adapter listeners for parity*/
        paritySpinner.setOnItemSelectedListener(new MyOnParitySelectedListener());
        /*set the adapter listeners for flow control*/
        flowSpinner.setOnItemSelectedListener(new MyOnFlowSelectedListener());
    }
    
    
    public class MyOnBaudSelectedListener implements OnItemSelectedListener {    
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) 
		{      
			//Toast.makeText(parent.getContext(), "The planet is " +   parent.getItemAtPosition(pos).toString(), 
			//			 Toast.LENGTH_LONG).show();    
			
			baudRate = Integer.parseInt(parent.getItemAtPosition(pos).toString());
			
			}
		
			public void onNothingSelected(AdapterView<?> parent) 
				{      // Do nothing.    }}
		}
    }
    
    public class MyOnStopSelectedListener implements OnItemSelectedListener {    
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) 
		{      
			//Toast.makeText(parent.getContext(), "The planet is " +   parent.getItemAtPosition(pos).toString(), 
			//			 Toast.LENGTH_LONG).show();    
			
			stopBit = (byte)Integer.parseInt(parent.getItemAtPosition(pos).toString());
			
			}
		
			public void onNothingSelected(AdapterView<?> parent) 
				{      // Do nothing.    }}
		}
    }
    
    public class MyOnDataSelectedListener implements OnItemSelectedListener {    
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) 
		{      
			//Toast.makeText(parent.getContext(), "The planet is " +   parent.getItemAtPosition(pos).toString(), 
			//			 Toast.LENGTH_LONG).show();    
			
			dataBit = (byte)Integer.parseInt(parent.getItemAtPosition(pos).toString());
			
			}
		
			public void onNothingSelected(AdapterView<?> parent) 
				{      // Do nothing.    }}
		}
    }
    
    public class MyOnParitySelectedListener implements OnItemSelectedListener {    
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) 
		{      
			//Toast.makeText(parent.getContext(), "The planet is " +   parent.getItemAtPosition(pos).toString(), 
			//			 Toast.LENGTH_LONG).show();    
			
			String parityString = new String(parent.getItemAtPosition(pos).toString());
			if(parityString.compareTo("none") == 0)
			{
				parity = 0;
			}
			
			if(parityString.compareTo("odd") == 0)
			{
				parity = 1;
			}

			if(parityString.compareTo("even") == 0)
			{
				parity = 2;
			}
			
			if(parityString.compareTo("mark") == 0)
			{
				parity = 3;
			}
			
			if(parityString.compareTo("space") == 0)
			{
				parity = 4;
			}
		}
		
		
			public void onNothingSelected(AdapterView<?> parent) 
				{      // Do nothing.    }}
		}
    }
    
    public class MyOnFlowSelectedListener implements OnItemSelectedListener {    
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) 
		{      
			//Toast.makeText(parent.getContext(), "The planet is " +   parent.getItemAtPosition(pos).toString(), 
			//			 Toast.LENGTH_LONG).show();    
			
			String flowString = new String(parent.getItemAtPosition(pos).toString());
			if(flowString.compareTo("none")==0)
			{
				flowControl = 0;
			}
			
			if(flowString.compareTo("CTS/RTS")==0)
			{
				flowControl = 1;
			}
			
		}
		
			public void onNothingSelected(AdapterView<?> parent) 
				{      // Do nothing.    }}
		}
    }
    
   
    
    @Override
    protected void onResume() {
        // Ideally should implement onResume() and onPause()
        // to take appropriate action when the activity looses focus
        super.onResume();
        uartInterface.ResumeAccessory();
    }

    @Override
    protected void onPause() {
        // Ideally should implement onResume() and onPause()
        // to take appropriate action when the activity looses focus
        super.onPause();
        
       
  
    }
    
    @Override 
    protected void onDestroy(){
    	
    	uartInterface.DestroyAccessory();
    	super.onDestroy();
    }
    
    
    final Handler handler =  new Handler()
    {
    	@Override 
    	public void handleMessage(Message msg)
    	{
    		if(actualNumBytes[0] != 0x00)
    		{
    			readText.append(String.copyValueOf(readBuffer, 0, actualNumBytes[0]));
    			
    			
    		}
    		
    		
    	}
    	
    };
	
	
	/*usb input data handler*/
	private class handler_thread  extends Thread {
		Handler mHandler;
		/*constructor*/
		handler_thread(Handler h ){
			mHandler = h;
			
		}
		
		public void run()
		{
			
			while(true)
			{
				Message msg = mHandler.obtainMessage();
				try{	
					Thread.sleep(50);
				} catch(InterruptedException e){}
				
				status = uartInterface.ReadData((byte)64, readBuffer, actualNumBytes);
				
				mHandler.sendMessage(msg);
				
				
				

			}
		}
	}
}
